@@ -2360,6 +2360,7 @@ nsGfxScrollFrameInner::LayoutScrollbars(nsBoxLayoutState& aState,
   scrollable->SetLineHeight(fontHeight);
 
   if (mVScrollbarBox) {
+    NS_PRECONDITION(mVScrollbarBox->IsBoxFrame(), "Must be a box frame!");
     if (!mHasVerticalScrollbar) {
       SetAttribute(mVScrollbarBox, nsXULAtoms::curpos, 0);
     }
@@ -2378,6 +2379,7 @@ nsGfxScrollFrameInner::LayoutScrollbars(nsBoxLayoutState& aState,
   }
     
   if (mHScrollbarBox) {
+    NS_PRECONDITION(mHScrollbarBox->IsBoxFrame(), "Must be a box frame!");
     if (!mHasHorizontalScrollbar) {
       SetAttribute(mHScrollbarBox, nsXULAtoms::curpos, 0);
     }
@@ -2397,6 +2399,7 @@ nsGfxScrollFrameInner::LayoutScrollbars(nsBoxLayoutState& aState,
 
   // place the scrollcorner
   if (mScrollCornerBox) {
+    NS_PRECONDITION(mScrollCornerBox->IsBoxFrame(), "Must be a box frame!");
     nsRect r(0, 0, 0, 0);
     if (aContentArea.x != aScrollArea.x) {
       // scrollbar (if any) on left
