@@ -83,6 +83,8 @@
 #include "nsIEventStateManager.h"
 #include "nsITimerInternal.h"
 #include "nsContentUtils.h"
+#include "nsIEventQueueService.h"
+#include "nsIServiceManager.h"
 
 #define NS_MENU_POPUP_LIST_INDEX   0
 
@@ -90,6 +92,8 @@
 #define NSCONTEXTMENUISMOUSEUP 1
 #endif
 
+static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
+
 static PRInt32 gEatMouseMove = PR_FALSE;
 
 nsMenuDismissalListener* nsMenuFrame::sDismissalListener = nsnull;
@@ -140,7 +144,6 @@ nsMenuFrame::Release(void)
 // QueryInterface
 //
 NS_INTERFACE_MAP_BEGIN(nsMenuFrame)
-  NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   NS_INTERFACE_MAP_ENTRY(nsIMenuFrame)
   NS_INTERFACE_MAP_ENTRY(nsIScrollableViewProvider)
 NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
@@ -176,6 +179,37 @@ nsMenuFrame::SetParent(const nsIFrame* aParent)
   return NS_OK;
 }
 
+struct nsASyncMenuInitialization : public PLEvent
+{
+  nsASyncMenuInitialization(nsIFrame* aFrame)
+    : mWeakFrame(aFrame)
+  {
+  }
+
+  void HandleEvent() {
+    if (mWeakFrame.IsAlive()) {
+      nsIMenuFrame* imenu = nsnull;
+      CallQueryInterface(mWeakFrame.GetFrame(), &imenu);
+      if (imenu) {
+        nsMenuFrame* menu = NS_STATIC_CAST(nsMenuFrame*, imenu);
+        menu->UpdateMenuType(menu->GetPresContext());
+      }
+    }
+  }
+
+  nsWeakFrame mWeakFrame;
+};
+
+static void* PR_CALLBACK HandleASyncMenuInitialization(PLEvent* aEvent)
+{
+  NS_STATIC_CAST(nsASyncMenuInitialization*, aEvent)->HandleEvent();
+  return nsnull;
+}
+static void PR_CALLBACK DestroyASyncMenuInitialization(PLEvent* aEvent)
+{
+  delete NS_STATIC_CAST(nsASyncMenuInitialization*, aEvent);
+}
+
 NS_IMETHODIMP
 nsMenuFrame::Init(nsPresContext*  aPresContext,
                      nsIContent*      aContent,
@@ -187,6 +221,11 @@ nsMenuFrame::Init(nsPresContext*  aPresContext,
 
   nsresult  rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
+  // Set up a mediator which can be used for callbacks on this frame.
+  mTimerMediator = new nsMenuTimerMediator(this);
+  if (NS_UNLIKELY(!mTimerMediator))
+    return NS_ERROR_OUT_OF_MEMORY;
+
   nsIFrame* currFrame = aParent;
   while (!mMenuParent && currFrame) {
     // Set our menu parent.
@@ -195,9 +234,6 @@ nsMenuFrame::Init(nsPresContext*  aPresContext,
     currFrame = currFrame->GetParent();
   }
 
-  // Do the type="checkbox" magic
-  UpdateMenuType(aPresContext);
-
   //load the display strings for the keyboard accelerators, but only once
   if (gRefCnt++ == 0) {
     
@@ -234,6 +270,24 @@ nsMenuFrame::Init(nsPresContext*  aPresContext,
   
   BuildAcceleratorText();
   
+  nsCOMPtr<nsIEventQueueService> eventService =
+    do_GetService(kEventQueueServiceCID);
+  NS_ENSURE_TRUE(eventService, PR_FALSE);
+  nsCOMPtr<nsIEventQueue> eventQueue;
+    eventService->GetThreadEventQueue(PR_GetCurrentThread(),
+                                      getter_AddRefs(eventQueue));
+  if (eventQueue) {
+    nsASyncMenuInitialization* initialization =
+      new nsASyncMenuInitialization(this);
+    if (initialization) {
+      PL_InitEvent(initialization, nsnull,
+                   ::HandleASyncMenuInitialization,
+                   ::DestroyASyncMenuInitialization);
+      if (NS_FAILED(eventQueue->PostEvent(initialization))) {
+        PL_DestroyEvent(initialization);
+      }
+    }
+  }
   return rv;
 }
 
@@ -284,12 +338,16 @@ nsMenuFrame::SetInitialChildList(nsPresContext* aPresContext,
       nsIMenuParent *menuPar;
       CallQueryInterface(frame, &menuPar);
       if (menuPar) {
-        // Remove this frame from the list and place it in the other list.
-        frames.RemoveFrame(frame);
-        mPopupFrames.AppendFrame(this, frame);
-        nsIFrame* first = frames.FirstChild();
-        rv = nsBoxFrame::SetInitialChildList(aPresContext, aListName, first);
-        return rv;
+        PRBool isMenuBar;
+        menuPar->IsMenuBar(isMenuBar);
+        if (!isMenuBar) {
+          // Remove this frame from the list and place it in the other list.
+          frames.RemoveFrame(frame);
+          mPopupFrames.AppendFrame(this, frame);
+          nsIFrame* first = frames.FirstChild();
+          rv = nsBoxFrame::SetInitialChildList(aPresContext, aListName, first);
+          return rv;
+        }
       }
       frame = frame->GetNextSibling();
     }
@@ -333,16 +391,30 @@ nsMenuFrame::DestroyPopupFrames(nsPresContext* aPresContext)
 NS_IMETHODIMP
 nsMenuFrame::Destroy(nsPresContext* aPresContext)
 {
+  // Kill our timer if one is active. This is not strictly necessary as
+  // the pointer to this frame will be cleared from the mediator, but
+  // this is done for added safety.
+  if (mOpenTimer) {
+    mOpenTimer->Cancel();
+  }
+
+  // Null out the pointer to this frame in the mediator wrapper so that it 
+  // doesn't try to interact with a deallocated frame.
+  mTimerMediator->ClearFrame();
+
+  nsWeakFrame weakFrame(this);
   // are we our menu parent's current menu item?
   if (mMenuParent) {
     nsIMenuFrame *curItem = mMenuParent->GetCurrentMenuItem();
     if (curItem == this) {
       // yes; tell it that we're going away
       mMenuParent->SetCurrentMenuItem(nsnull);
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
     }
   }
 
   UngenerateMenu();
+  NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
   DestroyPopupFrames(aPresContext);
   return nsBoxFrame::Destroy(aPresContext);
 }
@@ -378,6 +450,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
                              nsEventStatus*  aEventStatus)
 {
   NS_ENSURE_ARG_POINTER(aEventStatus);
+  nsWeakFrame weakFrame(this);
   if (*aEventStatus == nsEventStatus_eIgnore)
     *aEventStatus = nsEventStatus_eConsumeDoDefault;
   
@@ -405,6 +478,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
     // We have children.
     if ( isMenuBar || !mMenuParent ) {
       ToggleMenuState();
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
 
       if (!IsOpen() && mMenuParent) {
         // We closed up. The menu bar should always be
@@ -492,6 +566,8 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
 
     // Let the menu parent know we're the new item.
     mMenuParent->SetCurrentMenuItem(this);
+    NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
+    NS_ENSURE_TRUE(mMenuParent, NS_OK);
     
     // we need to check if we really became the current menu
     // item or not
@@ -517,7 +593,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
       nsCOMPtr<nsITimerInternal> ti = do_QueryInterface(mOpenTimer);
       ti->SetIdle(PR_FALSE);
 
-      mOpenTimer->InitWithCallback(this, menuDelay, nsITimer::TYPE_ONE_SHOT);
+      mOpenTimer->InitWithCallback(mTimerMediator, menuDelay, nsITimer::TYPE_ONE_SHOT);
 
     }
   }
@@ -527,9 +603,11 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsMenuFrame::ToggleMenuState()
-{  
+{
+  nsWeakFrame weakFrame(this);
   if (mMenuOpen) {
     OpenMenu(PR_FALSE);
+    NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
   }
   else {
     PRBool justRolledUp = PR_FALSE;
@@ -541,21 +619,28 @@ nsMenuFrame::ToggleMenuState()
       // from the same click. Otherwise, the user can't click on
       // a menubar item to toggle its submenu closed.
       OpenMenu(PR_FALSE);
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
       SelectMenu(PR_TRUE);
+      NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
+      NS_ENSURE_TRUE(mMenuParent, NS_OK);
       mMenuParent->SetActive(PR_FALSE);
     }
     else {
       if (mMenuParent) {
         mMenuParent->SetActive(PR_TRUE);
+        NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
       }
       OpenMenu(PR_TRUE);
     }
   }
+  NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
 
   if (mMenuParent) {
     // Make sure the current menu which is being toggled on
     // the menubar is highlighted
     mMenuParent->SetCurrentMenuItem(this);
+    NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
+    NS_ENSURE_TRUE(mMenuParent, NS_OK);
     // We've successfully prevented the same click from both
     // dismissing and reopening this menu. 
     // Clear the recent rollup state so we don't prevent
@@ -575,6 +660,7 @@ nsMenuFrame::SelectMenu(PRBool aActivateFlag)
 
   nsAutoString domEventToFire;
 
+  nsWeakFrame weakFrame(this);
   if (aActivateFlag) {
     // Highlight the menu.
     mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, NS_LITERAL_STRING("true"), PR_TRUE);
@@ -587,7 +673,9 @@ nsMenuFrame::SelectMenu(PRBool aActivateFlag)
     domEventToFire.AssignLiteral("DOMMenuItemInactive");
   }
 
-  FireDOMEvent(domEventToFire);
+  if (weakFrame.IsAlive()) {
+    FireChromeDOMEvent(domEventToFire);
+  }
   return NS_OK;
 }
 
@@ -677,7 +765,10 @@ nsMenuFrame::ActivateMenu(PRBool aActivateFlag)
 
   } else {
     if (mMenuOpen) {
-      menuPopup->FireDOMEvent(NS_LITERAL_STRING("DOMMenuInactive"));
+      nsWeakFrame weakFrame(this);
+      nsWeakFrame weakPopup(menuPopup);
+      menuPopup->FireChromeDOMEvent(NS_LITERAL_STRING("DOMMenuInactive"));
+      NS_ENSURE_TRUE(weakFrame.IsAlive() && weakPopup.IsAlive(), NS_OK);
     }
     nsIView* view = menuPopup->GetView();
     NS_ASSERTION(view, "View is gone, looks like someone forgot to rollup the popup!");
@@ -724,14 +815,20 @@ nsMenuFrame::OpenMenu(PRBool aActivateFlag)
   if (!mContent)
     return NS_OK;
 
+  nsWeakFrame weakFrame(this);
   if (aActivateFlag) {
     // Now that the menu is opened, we should have a menupopup child built.
     // Mark it as generated, which ensures a frame gets built.
     MarkAsGenerated();
+    NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
 
     mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
   }
-  else mContent->UnsetAttr(kNameSpaceID_None, nsXULAtoms::open, PR_TRUE);
+  else {
+    mContent->UnsetAttr(kNameSpaceID_None, nsXULAtoms::open, PR_TRUE);
+  }
+
+  NS_ENSURE_TRUE(weakFrame.IsAlive(), NS_OK);
   OpenMenuInternal(aActivateFlag);
 
   return NS_OK;
@@ -745,9 +842,11 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
   if (!mIsMenu)
     return;
 
+  nsWeakFrame weakFrame(this);
+
   if (aActivateFlag) {
     // Execute the oncreate handler
-    if (!OnCreate())
+    if (!OnCreate() || !weakFrame.IsAlive())
       return;
 
     mCreateHandlerSucceeded = PR_TRUE;
@@ -758,18 +857,22 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
     
     // XXX Only have this here because of RDF-generated content.
     MarkAsGenerated();
+    ENSURE_TRUE(weakFrame.IsAlive());
 
     nsIFrame* frame = mPopupFrames.FirstChild();
     nsMenuPopupFrame* menuPopup = (nsMenuPopupFrame*)frame;
     
+    PRBool wasOpen = mMenuOpen;
     mMenuOpen = PR_TRUE;
 
     if (menuPopup) {
+      nsWeakFrame weakMenuPopup(frame);
       // inherit whether or not we're a context menu from the parent
       if ( mMenuParent ) {
         PRBool parentIsContextMenu = PR_FALSE;
         mMenuParent->GetIsContextMenu(parentIsContextMenu);
         menuPopup->SetIsContextMenu(parentIsContextMenu);
+        ENSURE_TRUE(weakFrame.IsAlive());
       }
 
       // Install a keyboard navigation listener if we're the root of the menu chain.
@@ -779,12 +882,16 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
 
       if (mMenuParent && onMenuBar)
         mMenuParent->InstallKeyboardNavigator();
-      else if (!mMenuParent)
+      else if (!mMenuParent) {
+        ENSURE_TRUE(weakMenuPopup.IsAlive());
         menuPopup->InstallKeyboardNavigator();
+      }
       
       // Tell the menu bar we're active.
-      if (mMenuParent)
+      if (mMenuParent) {
         mMenuParent->SetActive(PR_TRUE);
+        ENSURE_TRUE(weakFrame.IsAlive());
+      }
 
       nsIContent* menuPopupContent = menuPopup->GetContent();
 
@@ -809,8 +916,11 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
 
       nsBoxLayoutState state(mPresContext);
 
-      // if height never set we need to do an initial reflow.
-      if (mLastPref.height == -1)
+      // If the menu popup was not open, do a reflow.  This is either the
+      // initial reflow for a brand-new popup, or a subsequent reflow for
+      // a menu that was deactivated and needs to be brought back to its
+      // active dimensions.
+      if (!wasOpen)
       {
          menuPopup->MarkDirty(state);
 
@@ -836,12 +946,14 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
       }
 
       ActivateMenu(PR_TRUE);
+      ENSURE_TRUE(weakFrame.IsAlive());
 
       nsIMenuParent *childPopup = nsnull;
       CallQueryInterface(frame, &childPopup);
       UpdateDismissalListener(childPopup);
 
       OnCreated();
+      ENSURE_TRUE(weakFrame.IsAlive());
     }
 
     // Set the focus back to our view's widget.
@@ -853,7 +965,7 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
 
     // Close the menu. 
     // Execute the ondestroy handler, but only if we're actually open
-    if ( !mCreateHandlerSucceeded || !OnDestroy() )
+    if ( !mCreateHandlerSucceeded || !OnDestroy() || !weakFrame.IsAlive())
       return;
 
     // Set the focus back to our view's widget.
@@ -868,6 +980,7 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
     // Make sure we clear out our own items.
     if (menuPopup) {
       menuPopup->SetCurrentMenuItem(nsnull);
+      ENSURE_TRUE(weakFrame.IsAlive());
       menuPopup->KillCloseTimer();
 
       PRBool onMenuBar = PR_TRUE;
@@ -893,6 +1006,7 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
     }
 
     ActivateMenu(PR_FALSE);
+    ENSURE_TRUE(weakFrame.IsAlive());
     // XXX hack: ensure that mMenuOpen is set to false, in case where
     // there is actually no popup. because ActivateMenu() will return 
     // early without setting it. It could be that mMenuOpen is true
@@ -903,6 +1017,7 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
     mMenuOpen = PR_FALSE;
 
     OnDestroyed();
+    ENSURE_TRUE(weakFrame.IsAlive());
 
     if (nsMenuFrame::sDismissalListener)
       nsMenuFrame::sDismissalListener->EnableListener(PR_TRUE);
@@ -1280,7 +1395,7 @@ nsMenuFrame::IsMenu()
   return mIsMenu;
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::Notify(nsITimer* aTimer)
 {
   // Our timer has fired.
@@ -1337,9 +1452,12 @@ nsMenuFrame::UpdateMenuType(nsPresContext* aPresContext)
     mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, mGroupName);
   } 
   else {
-    if (mType != eMenuType_Normal)
+    if (mType != eMenuType_Normal) {
+      nsWeakFrame weakFrame(this);
       mContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::checked,
                           PR_TRUE);
+      ENSURE_TRUE(weakFrame.IsAlive());
+    }
     mType = eMenuType_Normal;
   }
   UpdateMenuSpecialState(aPresContext);
@@ -1446,7 +1564,9 @@ nsMenuFrame::BuildAcceleratorText()
   AddStateBits(NS_STATE_ACCELTEXT_IS_DERIVED);
 
   // If anything below fails, just leave the accelerator text blank.
+  nsWeakFrame weakFrame(this);
   mContent->UnsetAttr(kNameSpaceID_None, nsXULAtoms::acceltext, PR_FALSE);
+  ENSURE_TRUE(weakFrame.IsAlive());
 
   // See if we have a key node and use that instead.
   nsAutoString keyValue;
@@ -1570,6 +1690,7 @@ nsMenuFrame::BuildAcceleratorText()
 void
 nsMenuFrame::Execute(nsGUIEvent *aEvent)
 {
+  nsWeakFrame weakFrame(this);
   // flip "checked" state if we're a checkbox menu, or an un-checked radio menu
   if (mType == eMenuType_Checkbox || (mType == eMenuType_Radio && !mChecked)) {
     nsAutoString value;
@@ -1578,10 +1699,12 @@ nsMenuFrame::Execute(nsGUIEvent *aEvent)
       if (mChecked) {
         mContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::checked,
                             PR_TRUE);
+        ENSURE_TRUE(weakFrame.IsAlive());
       }
       else {
         mContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::checked, NS_LITERAL_STRING("true"),
                           PR_TRUE);
+        ENSURE_TRUE(weakFrame.IsAlive());
       }        
       /* the AttributeChanged code will update all the internal state */
     }
@@ -1599,6 +1722,7 @@ nsMenuFrame::Execute(nsGUIEvent *aEvent)
 
   // Deselect ourselves.
   SelectMenu(PR_FALSE);
+  ENSURE_TRUE(weakFrame.IsAlive());
 
   // Now hide all of the open menus.
   if (mMenuParent) {
@@ -1634,24 +1758,15 @@ nsMenuFrame::Execute(nsGUIEvent *aEvent)
   // associated view manager on exit from this function.
   // See bug 54233.
   nsCOMPtr<nsIViewManager> kungFuDeathGrip = mPresContext->GetViewManager();
-  // keep a reference so we can safely use this after dispatching the DOM event
   nsCOMPtr<nsIPresShell> shell = mPresContext->GetPresShell();
-  nsIFrame* me = this;
   if (shell) {
     shell->HandleDOMEventWithTarget(mContent, &event, &status);
-    // shell may no longer be alive, don't use it here unless you keep a ref
+    ENSURE_TRUE(weakFrame.IsAlive());
   }
 
-  // XXX HACK. Just gracefully exit if the node has been removed, e.g., window.close()
-  // was executed.
-  nsIFrame* primary = nsnull;
-  if (shell) shell->GetPrimaryFrameFor(content, &primary);
-
-  // Now properly close them all up.
-  if (content->GetDocument() &&     // <-- HACK IS HERE. ICK.
-      (primary == me) && mMenuParent)
+  if (mMenuParent) {
     mMenuParent->DismissChain();
-  // END HACK
+  }
 
   // Re-enable rollup events on this menu.
   if ( nsMenuFrame::sDismissalListener ) {
@@ -1694,7 +1809,7 @@ nsMenuFrame::OnCreate()
 
     PRUint32 count = child->GetChildCount();
     for (PRUint32 i = 0; i < count; i++) {
-      nsIContent *grandChild = child->GetChildAt(i);
+      nsCOMPtr<nsIContent> grandChild = child->GetChildAt(i);
 
       if (grandChild->Tag() == nsXULAtoms::menuitem) {
         // See if we have a command attribute.
@@ -1914,6 +2029,51 @@ nsMenuFrame::UpdateDismissalListener(nsIMenuParent* aMenuParent)
   nsMenuFrame::sDismissalListener->SetCurrentMenuParent(aMenuParent);
 }
 
+struct nsASyncMenuGeneration : public PLEvent
+{
+  nsASyncMenuGeneration(nsIFrame* aFrame)
+    : mWeakFrame(aFrame)
+  {
+    nsIContent* content = aFrame ? aFrame->GetContent() : nsnull;
+    mDocument = content ? content->GetCurrentDoc() : nsnull;
+    if (mDocument) {
+      mDocument->BlockOnload();
+    }
+  }
+
+  void HandleEvent() {
+    nsIFrame* frame = mWeakFrame.GetFrame();
+    if (frame) {
+      PRBool collapsed = PR_FALSE;
+      nsBoxLayoutState state(frame->GetPresContext());
+      frame->IsCollapsed(state, collapsed);
+      if (!collapsed) {
+        nsIMenuFrame* imenu = nsnull;
+        CallQueryInterface(frame, &imenu);
+        if (imenu) {
+          imenu->MarkAsGenerated();
+        }
+      }
+    }
+    if (mDocument) {
+      mDocument->UnblockOnload();
+    }
+  }
+
+  nsWeakFrame           mWeakFrame;
+  nsCOMPtr<nsIDocument> mDocument;
+};
+
+static void* PR_CALLBACK HandleASyncMenuGeneration(PLEvent* aEvent)
+{
+  NS_STATIC_CAST(nsASyncMenuGeneration*, aEvent)->HandleEvent();
+  return nsnull;
+}
+static void PR_CALLBACK DestroyASyncMenuGeneration(PLEvent* aEvent)
+{
+  delete NS_STATIC_CAST(nsASyncMenuGeneration*, aEvent);
+}
+
 PRBool
 nsMenuFrame::SizeToPopup(nsBoxLayoutState& aState, nsSize& aSize)
 {
@@ -1927,10 +2087,32 @@ nsMenuFrame::SizeToPopup(nsBoxLayoutState& aState, nsSize& aSize)
     if (tmpSize.width == -1 && flex == 0) {
       nsIFrame* frame = mPopupFrames.FirstChild();
       if (!frame) {
-        MarkAsGenerated();
-        frame = mPopupFrames.FirstChild();
-        // No child - just return
-        if (!frame) return PR_FALSE;
+        nsCOMPtr<nsIContent> child;
+        GetMenuChildrenElement(getter_AddRefs(child));
+        if (child) {
+          nsAutoString genVal;
+          child->GetAttr(kNameSpaceID_None, nsXULAtoms::menugenerated, genVal);
+          if (genVal.IsEmpty()) {
+            nsCOMPtr<nsIEventQueueService> eventService =
+              do_GetService(kEventQueueServiceCID);
+            NS_ENSURE_TRUE(eventService, PR_FALSE);
+            nsCOMPtr<nsIEventQueue> eventQueue;
+              eventService->GetThreadEventQueue(PR_GetCurrentThread(),
+                                                getter_AddRefs(eventQueue));
+            if (eventQueue) {
+              nsASyncMenuGeneration* gen = new nsASyncMenuGeneration(this);
+              if (gen) {
+                PL_InitEvent(gen, nsnull,
+                             ::HandleASyncMenuGeneration,
+                             ::DestroyASyncMenuGeneration);
+                if (NS_FAILED(eventQueue->PostEvent(gen))) {
+                  PL_DestroyEvent(gen);
+                }
+              }
+            }
+          }
+        }
+        return PR_FALSE;
       }
 
       NS_ASSERTION(frame->IsBoxFrame(), "popupChild is not box!!");
@@ -2082,3 +2264,42 @@ nsMenuFrame::GetContextMenu()
 
   return nsnull;
 }
+
+// nsMenuTimerMediator implementation.
+NS_IMPL_ISUPPORTS1(nsMenuTimerMediator, nsITimerCallback)
+
+/**
+ * Constructs a wrapper around an nsMenuFrame.
+ * @param aFrame nsMenuFrame to create a wrapper around.
+ */
+nsMenuTimerMediator::nsMenuTimerMediator(nsMenuFrame *aFrame) :
+  mFrame(aFrame)
+{
+  NS_ASSERTION(mFrame, "Must have frame");
+}
+
+nsMenuTimerMediator::~nsMenuTimerMediator()
+{
+}
+
+/**
+ * Delegates the notification to the contained frame if it has not been destroyed.
+ * @param aTimer Timer which initiated the callback.
+ * @return NS_ERROR_FAILURE if the frame has been destroyed.
+ */
+NS_IMETHODIMP nsMenuTimerMediator::Notify(nsITimer* aTimer)
+{
+  if (!mFrame)
+    return NS_ERROR_FAILURE;
+
+  return mFrame->Notify(aTimer);
+}
+
+/**
+ * Clear the pointer to the contained nsMenuFrame. This should be called
+ * when the contained nsMenuFrame is destroyed.
+ */
+void nsMenuTimerMediator::ClearFrame()
+{
+  mFrame = nsnull;
+}
